local class = require("silo.lib.class")
local EventBus = class("silo.EventBus")

function EventBus:new()
  self.listeners = {}
end

function EventBus:fire(event, ...)
  if self.listeners[event] ~= nil then
    for _, listener in pairs(self.listeners[event]) do
      listener(...)
    end
  end
end

function EventBus:register(event, listener)
  if self.listeners[event] == nil then
    self.listeners[event] = { listener }
    return 1
  else
    table.insert(self.listeners[event], listener)
    return #self.listeners[event]
  end
end

function EventBus:unregister(event, id)
  if self.listeners[event] ~= nil then
    table.remove(self.listeners[event], id)
  end
end

function EventBus:handleOSEvents(ctx)
  ctx:addTask(function()
    while true do
      local ev = table.pack(os.pullEvent())
      self:handleOSEvent(ev)
    end
  end)
end

function EventBus:handleOSEvent(ev)
  local event = ev[1]
  table.remove(ev, 1)
  self:fire("event." .. event, table.unpack(ev))
end

return EventBus
